@@ -6,6 +6,7 @@ from collections import OrderedDict
 from fontTools.misc.xmlWriter import XMLWriter
 import fontFeatures
 from io import BytesIO
+import warnings
 
 
 def glyph(x):
@@ -44,19 +45,32 @@ class GTableUnparser:
 
     def _unparse_lookups(self, slr, inputs, in_lookups=None):
         lookups = []
+        note = None
         if in_lookups:
             lookups = in_lookups
+        indices = [sl.SequenceIndex for sl in slr]
+        if indices != list(sorted(indices)):
+            # https://github.com/adobe-type-tools/afdko/issues/1167
+            note = (
+                f"Out-of-order lookup application in lookup {self.currentLookup}\n"
+                f" A chaining rule calls lookups in the order {indices}\n"
+                " This cannot be directly expressed in Adobe FEA syntax;\n"
+                "  asFea() output will be incorrect"
+            )
+            warnings.warn(note)
         for sl in slr:
             if len(lookups) <= sl.SequenceIndex:
                 lookups.extend([None] * (1 + sl.SequenceIndex - len(lookups)))
             if not lookups[sl.SequenceIndex]:
                 lookups[sl.SequenceIndex] = []
+            if sl.LookupListIndex >= len(self.lookups):
+                raise ValueError("Lookups unparsed out of order: unparse %i first!" % sl.LookupListIndex)
             rr = fontFeatures.RoutineReference(routine=self.lookups[sl.LookupListIndex])
             lookups[sl.SequenceIndex].append(rr)
         if len(lookups) < len(inputs):
             lookups.extend([None] * (len(inputs) - len(lookups)))
         assert len(lookups) == len(inputs)
-        return lookups
+        return lookups, note
 
     def _invertClassDef(self, a, font):
         classes = {}
@@ -117,9 +131,7 @@ class GTableUnparser:
                     self.fontFeatures.addFeature(
                         featureTag,
                         [
-                            fontFeatures.RoutineReference(
-                                routine=self.lookups[lookupIdx]
-                            )
+                            self.lookups[lookupIdx]
                         ],
                     )
                 lookups.append(lookupIdx)
@@ -211,6 +223,8 @@ class GTableUnparser:
             xt.SubTable = [xt.ExtSubTable]
             xt.LookupType = xt.ExtSubTable.LookupType
             xt.LookupFlag = lookup.LookupFlag
+            if hasattr(lookup, "MarkFilteringSet"):
+                xt.MarkFilteringSet = lookup.MarkFilteringSet
             routine, deps = self.unparseLookup(xt, self.currentLookup)
             routines.append(routine)
             dependencies.extend(deps)
@@ -218,6 +232,7 @@ class GTableUnparser:
             routines=routines,
             name=self.getname("Extension" + self.gensym()),
         )
+        self._fix_flags(extension, lookup)
         return extension, dependencies
 
     def getDebugInfo(self, table, ix):
@@ -259,13 +274,16 @@ class GTableUnparser:
             glyphs = [g for g in classDefs.keys() if classDefs[g] == mat]
             routine.markAttachmentSet = glyphs
         if lookup.LookupFlag & 0x10 and hasattr(lookup, "MarkFilteringSet"):
-            # It might not have one if it's an Extension lookup
             routine.markFilteringSet = (
                 self.font["GDEF"]
                 .table.MarkGlyphSetsDef.Coverage[lookup.MarkFilteringSet]
                 .glyphs
             )
 
+        if routine.flags & 0x10:
+            assert routine.markFilteringSet is not None
+
+
     def unparseContextual(self, lookup):
         """Handles a generic contextual lookup, in various formats."""
         b = fontFeatures.Routine(
@@ -293,15 +311,16 @@ class GTableUnparser:
             for subrule in getattr(subrulesets, ruleattr):
                 lookups = []
                 allinput = [glyph(x) for x in ([input_] + subrule.Input)]
-                lookups = self._unparse_lookups(getattr(subrule, lookupattr), allinput)
-                b.addRule(
-                    fontFeatures.Chaining(
+                lookups, note = self._unparse_lookups(getattr(subrule, lookupattr), allinput)
+                rule = fontFeatures.Chaining(
                         allinput,
                         lookups=lookups,
                         address=self.currentLookup,
                         flags=lookup.LookupFlag,
                     )
-                )
+                if note:
+                    rule.note = note
+                b.addRule(rule)
         return
 
     def _unparse_contextual_format2(self, sub, b, lookup):
@@ -339,9 +358,8 @@ class GTableUnparser:
                 inputs = [glyph(x) for x in ([input_] + subrule.Input)]
                 prefix = [glyph(x) for x in reversed(subrule.Backtrack)]
                 suffix = [glyph(x) for x in subrule.LookAhead]
-                lookups = self._unparse_lookups(getattr(subrule, lookupattr), inputs)
-                b.addRule(
-                    fontFeatures.Chaining(
+                lookups, note = self._unparse_lookups(getattr(subrule, lookupattr), inputs)
+                rule = fontFeatures.Chaining(
                         inputs,
                         prefix,
                         suffix,
@@ -349,7 +367,9 @@ class GTableUnparser:
                         address=self.currentLookup,
                         flags=lookup.LookupFlag,
                     )
-                )
+                if note:
+                    rule.note = note
+                b.addRule(rule)
 
     def _unparse_contextual_chain_format2(self, sub, b, lookup, chain=True):
         if chain:
@@ -394,9 +414,8 @@ class GTableUnparser:
                 else:
                     prefix, suffix = [], []
                     input_ = [inputclass] + [inputs[x] for x in r.Class]
-                lookups = self._unparse_lookups(getattr(r, lookupattr), input_)
-                b.addRule(
-                    fontFeatures.Chaining(
+                lookups, note = self._unparse_lookups(getattr(r, lookupattr), input_)
+                rule = fontFeatures.Chaining(
                         input_,
                         prefix,
                         suffix,
@@ -404,7 +423,9 @@ class GTableUnparser:
                         address=self.currentLookup,
                         flags=lookup.LookupFlag,
                     )
-                )
+                if note:
+                    rule.note = note
+                b.addRule(rule)
 
     def _unparse_contextual_chain_format3(self, sub, b, lookup, chain=True):
         lookupattr = self._attrs["lookup"]
@@ -421,9 +442,8 @@ class GTableUnparser:
             for coverage in sub.Coverage:
                 inputs.append(coverage.glyphs)
 
-        lookups = self._unparse_lookups(getattr(sub, lookupattr), inputs)
-        b.addRule(
-            fontFeatures.Chaining(
+        lookups, note = self._unparse_lookups(getattr(sub, lookupattr), inputs)
+        rule = fontFeatures.Chaining(
                 inputs,
                 prefix,
                 suffix,
@@ -431,4 +451,6 @@ class GTableUnparser:
                 address=self.currentLookup,
                 flags=lookup.LookupFlag,
             )
-        )
+        if note:
+            rule.note = note
+        b.addRule(rule)
